/*  																													
	    			       Jaguar-jCompany Developer Suite.																		
			    		        Powerlogic 2015-2020.
			    		    
		Please read licensing information in your installation directory.
		Contact Powerlogic for more information or contribute with this project. 
			site...: www.powerlogic.org																								
			e-mail.: suporte@powerlogic.com.br
*/

package com.powerlogic.jcompany.core.messages;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.bind.annotation.XmlRootElement;

@XmlRootElement
public class PlcMessageMap implements Serializable {
	
	private static final long serialVersionUID = 1L;
	
	private Map<PlcMessageType, List<PlcMessageEntry>> messages = new HashMap<>();

	public Map<PlcMessageType, List<PlcMessageEntry>> getMessages()
	{
		return messages;
	}

	public void setMessages(Map<PlcMessageType, List<PlcMessageEntry>> messages)
	{
		this.messages = messages;
	}

	public void add(PlcMessageEntry message)
	{
		List<PlcMessageEntry> lista = getTypeList(message.getType());
		if (lista.contains(message)) {
			PlcMessageEntry messageAnterior = lista.get(lista.indexOf(message));
			messageAnterior.setMessage(message.getMessage());
			messageAnterior.setArgs(message.getArgs());
		} else {
			lista.add(message);
		}
	}

	public void addAll(PlcMessageMap messageMap)
	{
		for (Entry<PlcMessageType, List<PlcMessageEntry>> messages : messageMap.messages.entrySet())
		{
			List<PlcMessageEntry> lista = getTypeList(messages.getKey());
			for(PlcMessageEntry message : messages.getValue()) {
				if (lista.contains(message)) {
					PlcMessageEntry messageAnterior = lista.get(lista.indexOf(message));
					messageAnterior.setMessage(message.getMessage());
					messageAnterior.setArgs(message.getArgs());
				} else{
					lista.add(message);
				}
			}
		}
	}

	public void addAll(Collection<PlcMessageEntry> messages)
	{
		for (PlcMessageEntry message : messages)
		{
			add(message);
		}
	}

	public boolean contains(PlcMessageType type)
	{
		List<PlcMessageEntry> list = messages.get(type);
		return list != null && !list.isEmpty();
	}

	public List<PlcMessageEntry> get(PlcMessageType type)
	{
		List<PlcMessageEntry> list = messages.get(type);
		if (list == null || list.isEmpty())
		{
			return Collections.emptyList();
		}
		return new ArrayList<>(list);
	}

	public void clear()
	{
		messages.clear();
	}

	public void addMessage(IPlcMessageKey key, PlcMessageType type)
	{
		addMessage(key, type, null);
	}

	public void addMessage(IPlcMessageKey key, PlcMessageType type, String[] args)
	{
		add(new PlcMessageEntry(key, type, args));
	}

	protected List<PlcMessageEntry> getTypeList(PlcMessageType type)
	{
		List<PlcMessageEntry> list = messages.get(type);

		if (list == null)
		{
			messages.put(type, list = new LinkedList<PlcMessageEntry>());
		}
		return list;
	}
}
